# -*- coding: utf-8 -*-
import re

from django.utils.translation import ugettext_lazy as _

from .router import Patterns

HTTP_STATUS_CODES = (
    # Infomational
    (100, _('Continue')),
    (101, _('Switching Protocols')),
    (102, _('Processing (WebDAV)')),

    # Success
    (200, _('OK')),
    (201, _('Created')),
    (202, _('Accepted')),
    (203, _('Non-Authoritative Information')),
    (204, _('No Content')),
    (205, _('Reset Content')),
    (206, _('Partial Content')),
    (207, _('Multi-Status (WebDAV)')),

    # Redirection
    (300, _('Multiple Choices')),
    (301, _('Moved Permanently')),
    (302, _('Found')),
    (303, _('See Other')),
    (304, _('Not Modified')),
    (305, _('Use Proxy')),
    (306, _('Switch Proxy')),  # No longer used
    (307, _('Temporary Redirect')),

    # Client Error
    (400, _('Bad Request')),
    (401, _('Unauthorized')),
    (402, _('Payment Required')),
    (403, _('Forbidden')),
    (404, _('Not Found')),
    (405, _('Method Not Allowed')),
    (406, _('Not Acceptable')),
    (407, _('Proxy Authentication Required')),
    (408, _('Request Timeout')),
    (409, _('Conflict')),
    (410, _('Gone')),
    (411, _('Length Required')),
    (412, _('Precondition Failed')),
    (413, _('Request Entity Too Large')),
    (414, _('Request-URI Too Long')),
    (415, _('Unsupported Media Type')),
    (416, _('Requested Range Not Satisfiable')),
    (417, _('Expectation Failed')),
    (418, _('I\'m a teapot')),  # April Fools
    (422, _('Unprocessable Entity (WebDAV)')),
    (423, _('Locked (WebDAV)')),
    (424, _('Failed Dependency (WebDAV)')),
    (425, _('Unordered Collection')),
    (426, _('Upgrade Required')),
    (449, _('Retry With')),

    # Server Error
    (500, _('Internal Server Error')),
    (501, _('Not Implemented')),
    (502, _('Bad Gateway')),
    (503, _('Service Unavailable')),
    (504, _('Gateway Timeout')),
    (505, _('HTTP Version Not Supported')),
    (506, _('Variant Also Negotiates')),
    (507, _('Insufficient Storage (WebDAV)')),
    (509, _('Bandwidth Limit Exceeded')),
    (510, _('Not Extended')),
)


browsers = Patterns(
    ('Unknown', {}),
    # Browsers
    (r'AOL (?P<version>[\d+\.\d+]+)', 'AOL'),
    (
        r'Mozilla/(?P<mozilla_version>[-.\w]+) \(compatible; ( ?)MSIE (?P<msie_version>[-.\w]+); ' +
        r'( ?)( ?)America Online Browser (?P<version>[-.\w]+);',
        'AOL',
    ),
    (r'Camino/(?P<version>[-.\w]+)', 'Camino'),
    (r'Chrome/(?P<version>[-.\w]+)', 'Google Chrome'),
    (r'Firefox(/(?P<version>[-.\w]+)?)', 'Firefox'),
    (
        r'Mozilla/(?P<mozilla_version>[-.\w]+) \(compatible; ( ?)MSIE (?P<version>[-.\w]+); ' +
        r'( ?)( ?)(Win|Mac)',
        'Internet Explorer',
    ),
    (r'Konqueror/(?P<version>[-.\w]+)', 'Konqueror'),
    (r'Opera( |/)(?P<version>[-.\w]+)', 'Opera'),
    (r'OmniWeb(/(?P<version>[-.\w]+)?)', 'OmniWeb'),
    (r'Safari/(?P<version>[-.\w]+)', 'Safari'),
    (r'(Netscape([\d]?)|Navigator)/(?P<version>[-.\w]+)', 'Netscape'),
    (r'Wget/(?P<version>[-.\w]+)', 'Wget'),

    (r'Minefield(/(?P<version>[-.\w]+)?)', 'Firefox'),  # Firefox nightly trunk builds
    (r'Shiretoko(/(?P<version>[-.\w]+)?)', 'Firefox'),  # Firefox testing browser
    (r'GranParadiso(/(?P<version>[-.\w]+)?)', 'Firefox'),  # Firefox testing browser
    (r'Iceweasel(/(?P<version>[-.\w]+)?)', 'Firefox'),  # Debian re-branded firefox

    # RSS Reader
    (r'(NetNewsWire|NewsGatorOnline)/(?P<version>[-.\w]+)', 'NetNewsWire'),
    (r'Feedfetcher-Google', 'Google Reader'),

    # Bots
    (r'Googlebot', 'Google'),
    (r'Yahoo! Slurp', 'Yahoo'),
    (r'msnbot', 'MSN Bot'),
    (r'(Baiduspider|BaiduImagespider)', 'Baiduspider'),
    (r'Ask Jeeves', 'Ask Jeeves'),
    (r'FollowSite', 'FollowSite'),
    (r'WebAlta Crawler', 'WebAlta Crawler'),
    (r'ScoutJet', 'ScoutJet'),
    (r'SurveyBot', 'domaintools.com'),
    (r'Gigabot', 'Gigabot'),
    (r'Speedy Spider', 'entireweb'),
    (r'discobot', 'Discovery Engine'),
    (r'Purebot(/(?P<version>[-.\w]+)?);', 'Purity search'),
    (r'Yandex(/(?P<version>[-.\w]+)?)', 'Yandex'),
    (r'PostRank(/(?P<version>[-.\w]+)?)', 'PostRank'),
    (
        r'Mozilla/(?P<mozilla_version>[-.\w]+) \(compatible; DotBot/(?P<version>[-.\w]+); ' +
        r'http://www.dotnetdotcom.org/, crawler@dotnetdotcom.org\)',
        'Dotbot',
    ),
    (r'IrssiUrlLog(/(?P<version>[-.\w]+)?)', 'irssi'),
    (r'Linguee Bot \(http://www.linguee.com/bot; bot@linguee.com\)', 'Linguee'),
    (r'Sphider', 'Sphider'),

    # Other
    (r'Mediapartners-Google', 'Google Ads'),
    (r'Apple-PubSub', 'Apple-PubSub'),
    (r'Python-urllib', 'Python'),
)

engines = Patterns(
    None,
    (r'^https?:\/\/([\.\w]+)?yahoo.*(?:&|\?)p=(?P<keywords>[\+-_\w]+)', 'Yahoo'),
    (r'^https?:\/\/([\.\w]+)?google.*(?:&|\?)q=(?P<keywords>[\+-_\w]+)', 'Google'),
    (r'^https?:\/\/([\.\w]+)?bing.*(?:&|\?)q=(?P<keywords>[\+-_\w]+)', 'Bing'),
)


def get_verbose_name(class_name):
    '''
    Calculate the verbose_name by converting from InitialCaps to
    "lowercase with spaces".
    '''
    return re.sub(
        '(((?<=[a-z])[A-Z])|([A-Z](?![A-Z]|$)))',
        ' \\1',
        class_name,
    ).strip()



# -*- coding: utf-8 -*-
from datetime import timedelta

from dateutil.relativedelta import relativedelta
from django.core.management.base import BaseCommand, CommandError
from django.utils import timezone

from ...models import Request

DURATION_OPTIONS = {
    'hours': lambda amount: timezone.now() - timedelta(hours=amount),
    'days': lambda amount: timezone.now() - timedelta(days=amount),
    'weeks': lambda amount: timezone.now() - timedelta(weeks=amount),
    'months': lambda amount: timezone.now() + relativedelta(months=-amount),
    'years': lambda amount: timezone.now() + relativedelta(years=-amount),
}

try:
    # to keep backward Python 2 compatibility
    input = raw_input
except NameError:
    pass


class Command(BaseCommand):
    help = 'Purge old requests.'

    def add_arguments(self, parser):
        parser.add_argument(
            'amount',
            type=int,
        )
        parser.add_argument('duration')
        parser.add_argument(
            '--noinput',
            action='store_false',
            dest='interactive',
            default=True,
            help='Tells Django to NOT prompt the user for input of any kind.'
        )

    def handle(self, *args, **options):
        amount = options['amount']
        duration = options['duration']

        # Check we have the correct values
        if duration[-1] != 's':  # If its not plural, make it plural
            duration_plural = '{0}s'.format(duration)
        else:
            duration_plural = duration

        if duration_plural not in DURATION_OPTIONS:
            raise CommandError('Amount must be {0}'.format(', '.join(DURATION_OPTIONS)))

        qs = Request.objects.filter(time__lte=DURATION_OPTIONS[duration_plural](amount))
        count = qs.count()

        if count == 0:
            print('There are no requests to delete.')
            return

        if options.get('interactive'):
            confirm = input('''
You have requested a database reset.
This will IRREVERSIBLY DESTROY any
requests created before {0} {1} ago.
That is a total of {2} requests.
Are you sure you want to do this?

Type 'yes' to continue, or 'no' to cancel:'''.format(amount, duration, count))
        else:
            confirm = 'yes'

        if confirm == 'yes':
            qs.delete()
        else:
            print('Purge cancelled')



# -*- coding: utf-8 -*-
import re

from six import string_types


class RegexPattern(object):
    def __init__(self, regex, name=''):
        self.regex = re.compile(regex, re.UNICODE)
        self.name = name

    def resolve(self, string):
        match = self.regex.search(string)
        if match:
            return self.name, match.groupdict()


class Patterns(object):
    def __init__(self, unknown, *args):
        self.patterns = ()
        self.unknown = unknown

        for pattern in args:
            if isinstance(pattern, string_types):
                self.patterns += (RegexPattern(pattern),)
            else:
                self.patterns += (RegexPattern(*pattern),)

    def resolve(self, name):
        for pattern in self.patterns:
            match = pattern.resolve(name)
            if match:
                return match
        return self.unknown



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Request',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('response', models.SmallIntegerField(default=200, verbose_name='response', choices=[(100, 'Continue'), (101, 'Switching Protocols'), (102, 'Processing (WebDAV)'), (200, 'OK'), (201, 'Created'), (202, 'Accepted'), (203, 'Non-Authoritative Information'), (204, 'No Content'), (205, 'Reset Content'), (206, 'Partial Content'), (207, 'Multi-Status (WebDAV)'), (300, 'Multiple Choices'), (301, 'Moved Permanently'), (302, 'Found'), (303, 'See Other'), (304, 'Not Modified'), (305, 'Use Proxy'), (306, 'Switch Proxy'), (307, 'Temporary Redirect'), (400, 'Bad Request'), (401, 'Unauthorized'), (402, 'Payment Required'), (403, 'Forbidden'), (404, 'Not Found'), (405, 'Method Not Allowed'), (406, 'Not Acceptable'), (407, 'Proxy Authentication Required'), (408, 'Request Timeout'), (409, 'Conflict'), (410, 'Gone'), (411, 'Length Required'), (412, 'Precondition Failed'), (413, 'Request Entity Too Large'), (414, 'Request-URI Too Long'), (415, 'Unsupported Media Type'), (416, 'Requested Range Not Satisfiable'), (417, 'Expectation Failed'), (418, "I'm a teapot"), (422, 'Unprocessable Entity (WebDAV)'), (423, 'Locked (WebDAV)'), (424, 'Failed Dependency (WebDAV)'), (425, 'Unordered Collection'), (426, 'Upgrade Required'), (449, 'Retry With'), (500, 'Internal Server Error'), (501, 'Not Implemented'), (502, 'Bad Gateway'), (503, 'Service Unavailable'), (504, 'Gateway Timeout'), (505, 'HTTP Version Not Supported'), (506, 'Variant Also Negotiates'), (507, 'Insufficient Storage (WebDAV)'), (509, 'Bandwidth Limit Exceeded'), (510, 'Not Extended')])),
                ('method', models.CharField(default=b'GET', max_length=7, verbose_name='method')),
                ('path', models.CharField(max_length=255, verbose_name='path')),
                ('time', models.DateTimeField(auto_now_add=True, verbose_name='time')),
                ('is_secure', models.BooleanField(default=False, verbose_name='is secure')),
                ('is_ajax', models.BooleanField(default=False, help_text='Wheather this request was used via javascript.', verbose_name='is ajax')),
                ('ip', models.IPAddressField(verbose_name='ip address')),
                ('referer', models.URLField(max_length=255, null=True, verbose_name='referer', blank=True)),
                ('user_agent', models.CharField(max_length=255, null=True, verbose_name='user agent', blank=True)),
                ('language', models.CharField(max_length=255, null=True, verbose_name='language', blank=True)),
                ('user', models.ForeignKey(verbose_name='user', blank=True, to=settings.AUTH_USER_MODEL, null=True, on_delete=models.CASCADE)),
            ],
            options={
                'ordering': ('-time',),
                'verbose_name': 'request',
                'verbose_name_plural': 'requests',
            },
            bases=(models.Model,),
        ),
    ]



# -*- coding: utf-8 -*-
import datetime
import time

from django.db import models
from django.db.models import Q
from django.utils import timezone

from . import settings

QUERYSET_PROXY_METHODS = (
    'year',
    'month',
    'week',
    'day',
    'today',
    'this_week',
    'this_month',
    'this_year',
    'unique_visits',
    'attr_list',
    'search',
)


class RequestQuerySet(models.query.QuerySet):
    def year(self, year):
        return self.filter(time__year=year)

    def month(self, year=None, month=None, month_format='%b', date=None):
        if not date:
            try:
                if year and month:
                    date = datetime.date(*time.strptime(year + month, '%Y' + month_format)[:3])
                else:
                    raise TypeError('Request.objects.month() takes exactly 2 arguments')
            except ValueError:
                return
        # Truncate to date.
        if isinstance(date, datetime.datetime):
            date = date.date()
        return self.filter(time__year=date.year, time__month=date.month)

    def week(self, year, week):
        try:
            date = datetime.date(*time.strptime(year + '-0-' + week, '%Y-%w-%U')[:3])
        except ValueError:
            return

        # Calculate first and last day of week, for use in a date-range lookup.
        first_day = date
        last_day = date + datetime.timedelta(days=7)
        lookup_kwargs = {
            'time__gte': first_day,
            'time__lt': last_day,
        }

        return self.filter(**lookup_kwargs)

    def day(self, year=None, month=None, day=None, month_format='%b', day_format='%d', date=None):
        if not date:
            try:
                if year and month and day:
                    date = datetime.date(*time.strptime(year + month + day, '%Y' + month_format + day_format)[:3])
                else:
                    raise TypeError('Request.objects.day() takes exactly 3 arguments')
            except ValueError:
                return
        return self.filter(time__date=date)

    def today(self):
        return self.day(date=datetime.date.today())

    def this_year(self):
        return self.year(datetime.date.today().year)

    def this_month(self):
        return self.month(date=datetime.date.today())

    def this_week(self):
        today = datetime.date.today()
        return self.week(str(today.year), today.strftime('%U'))

    def unique_visits(self):
        return self.exclude(referer__startswith=settings.BASE_URL)

    def attr_list(self, name):
        return [getattr(item, name, None) for item in self if hasattr(item, name)]

    def search(self):
        return self.filter(Q(referer__contains='google') | Q(referer__contains='yahoo') | Q(referer__contains='bing'))


class RequestManager(models.Manager):
    def __getattr__(self, attr, *args, **kwargs):
        if attr in QUERYSET_PROXY_METHODS:
            return getattr(self.get_queryset(), attr, None)
        super(RequestManager, self).__getattr__(*args, **kwargs)

    def get_queryset(self):
        return RequestQuerySet(self.model)

    def active_users(self, **options):
        '''
        Returns a list of active users.

        Any arguments passed to this method will be
        given to timedelta for time filtering.

        Example:
        >>> Request.object.active_users(minutes=15)
        [<User: kylef>, <User: krisje8>]
        '''

        qs = self.filter(user__isnull=False)

        if options:
            time = timezone.now() - datetime.timedelta(**options)
            qs = qs.filter(time__gte=time)

        requests = qs.select_related('user').only('user')

        return set([request.user for request in requests])



# -*- coding: utf-8 -*-
from django import template

from ..models import Request

register = template.Library()


class ActiveUserNode(template.Node):
    def __init__(self, parser, token):
        tokens = token.contents.split()
        tag_name = tokens.pop(0)
        self.kwargs = {}
        self.as_varname = 'user_list'

        if len(tokens) not in (5, 2, 0):
            raise template.TemplateSyntaxError('Incorrect amount of arguments in the tag {0!r}'.format(tag_name))

        if len(tokens) == 5 and tokens[0] == 'in':
            tokens.pop(0)  # pop 'in' of tokens
            try:
                self.kwargs[str(tokens.pop(0))] = int(tokens.pop(0))
            except ValueError:
                raise template.TemplateSyntaxError('Invalid arguments for {0!r} template tag.'.format(tag_name))
        else:
            self.kwargs['minutes'] = 15

        if len(tokens) == 2 and tokens[0] == 'as':
            self.as_varname = tokens[1]

    def render(self, context):
        context[self.as_varname] = Request.objects.active_users(**self.kwargs)
        return ''


@register.tag
def active_users(parser, token):
    '''
    This template tag will get a list of active users based on time,
    if you do not supply a time to the tag, the default of 15 minutes
    will be used. With the 'as' clause you can supply what context
    variable you want the user list to be. There is also a 'in' clause,
    after in you would specify a amount and a duration. Such as 2 hours,
    of 10 minutes.

    Syntax::
        {% active_users in [amount] [duration] as [varname] %}
        {% active_users as [varname] %}
        {% active_users %}

    Example usage::
        {% load request_tag %}
        {% active_users in 10 minutes as user_list %}
        {% for user in user_list %}
            {{ user.get_username }}
        {% endfor %}
    '''
    return ActiveUserNode(parser, token)



# -*- coding: utf-8 -*-
from . import settings
from .models import Request
from .router import Patterns

try:
    # needed to support Django >= 1.10 MIDDLEWARE
    from django.utils.deprecation import MiddlewareMixin
except ImportError:
    # needed to keep Django <= 1.9 MIDDLEWARE_CLASSES
    MiddlewareMixin = object


class RequestMiddleware(MiddlewareMixin):
    def process_request(self, request):
        self._initial_http_body = request.body
    
    def process_response(self, request, response):
        if request.method.lower() not in settings.VALID_METHOD_NAMES:
            return response

        if response.status_code < 400 and settings.ONLY_ERRORS:
            return response

        ignore = Patterns(False, *settings.IGNORE_PATHS)
        if ignore.resolve(request.path[1:]):
            return response

        if request.is_ajax() and settings.IGNORE_AJAX:
            return response

        if request.META.get('REMOTE_ADDR') in settings.IGNORE_IP:
            return response

        ignore = Patterns(False, *settings.IGNORE_USER_AGENTS)
        if ignore.resolve(request.META.get('HTTP_USER_AGENT', '')):
            return response

        if getattr(request, 'user', False):
            if request.user.get_username() in settings.IGNORE_USERNAME:
                return response

        r = Request()
        r.from_http_request(request, response)

        return response



# Generated by Django 2.0.2 on 2019-08-05 09:37

from django.db import migrations, models
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('request', '0006_alter_request_method_default'),
    ]

    operations = [
        migrations.AddField(
            model_name='request',
            name='get_params',
            field=models.CharField(blank=True, max_length=512, null=True),
        ),
        migrations.AddField(
            model_name='request',
            name='post_params',
            field=jsonfield.fields.JSONField(blank=True, default='{}', null=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('request', '0004_alter_time_timezone_default'),
    ]

    operations = [
        migrations.AlterField(
            model_name='request',
            name='user',
            field=models.ForeignKey(blank=True, null=True, on_delete=models.SET_NULL, to=settings.AUTH_USER_MODEL, verbose_name='user'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('request', '0002_alter_request_ip'),
    ]

    operations = [
        migrations.AlterField(
            model_name='request',
            name='time',
            field=models.DateTimeField(auto_now_add=True, verbose_name='time', db_index=True),
        ),
    ]



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('request', '0005_alter_request_user'),
    ]

    operations = [
        migrations.AlterField(
            model_name='request',
            name='method',
            field=models.CharField(default='GET', max_length=7, verbose_name='method'),
        ),
    ]



# -*- coding: utf-8 -*-
from time import mktime

import django
from django.core.exceptions import ImproperlyConfigured
from django.db.models import Count
from django.utils.translation import ugettext
from django.utils.translation import ugettext_lazy as _

from . import settings
from .utils import get_verbose_name


class Modules(object):
    '''
    Set of :class:`.Module`.
    '''
    def load(self):
        '''
        Import and instanciate modules defined in
        ``settings.TRAFFIC_MODULES``.
        '''
        from importlib import import_module

        self._modules = ()
        for module_path in settings.TRAFFIC_MODULES:
            try:
                dot = module_path.rindex('.')
            except ValueError:
                raise ImproperlyConfigured('{0} isn\'t a traffic module'.format(module_path))
            traffic_module = module_path[:dot]
            traffic_classname = module_path[dot + 1:]

            try:
                mod = import_module(traffic_module)
            except ImportError as err:
                raise ImproperlyConfigured('Error importing module {0}: "{1}"'.format(traffic_module, err))

            try:
                traffic_class = getattr(mod, traffic_classname)
            except AttributeError:
                raise ImproperlyConfigured('Traffic module "{0}" does not define a "{1}" class'.format(
                    traffic_module,
                    traffic_classname,
                ))

            self._modules += (traffic_class(),)

    @property
    def modules(self):
        '''
        Get loaded modules, load them if isn't already made.
        '''
        if not hasattr(self, '_modules'):
            self.load()
        return self._modules

    def table(self, queries):
        '''
        Get a list of modules' counters.
        '''
        return tuple([(
            module.verbose_name_plural, [module.count(qs) for qs in queries]
        ) for module in self.modules])

    def graph(self, days):
        '''
        Get a list of modules' counters for all the given days.
        '''
        return tuple([{
            'data': [(mktime(day.timetuple()) * 1000, module.count(qs)) for day, qs in days],
            'label': str(ugettext(module.verbose_name_plural)),
        } for module in self.modules])


modules = Modules()


class Module(object):
    '''
    Base module class.
    '''
    def __init__(self):
        self.module_name = self.__class__.__name__

        if not hasattr(self, 'verbose_name'):
            self.verbose_name = get_verbose_name(self.module_name)
        if not hasattr(self, 'verbose_name_plural'):
            if django.VERSION < (1, 11):
                from django.utils.translation import string_concat
                self.verbose_name_plural = string_concat(self.verbose_name, 's')
            else:
                from django.utils.text import format_lazy
                self.verbose_name_plural = format_lazy('{}{}', self.verbose_name, 's')

    def count(self, qs):
        raise NotImplementedError('"count" isn\'t defined.')


class Ajax(Module):
    verbose_name_plural = _('Ajax')

    def count(self, qs):
        return qs.filter(is_ajax=True).count()


class NotAjax(Module):
    verbose_name = _('Not Ajax')
    verbose_name_plural = _('Not Ajax')

    def count(self, qs):
        return qs.filter(is_ajax=False).count()


class Error(Module):
    verbose_name = _('Error')
    verbose_name_plural = _('Errors')

    def count(self, qs):
        return qs.filter(response__gte=400).count()


class Error404(Module):
    verbose_name = _('Error 404')
    verbose_name_plural = _('Errors 404')

    def count(self, qs):
        return qs.filter(response=404).count()


class Hit(Module):
    verbose_name = _('Hit')
    verbose_name_plural = _('Hits')

    def count(self, qs):
        return qs.count()


class Search(Module):
    verbose_name = _('Search')
    verbose_name_plural = _('Searches')

    def count(self, qs):
        return qs.search().count()


class Secure(Module):
    verbose_name = _('Secure')
    verbose_name_plural = _('Secure')

    def count(self, qs):
        return qs.filter(is_secure=True).count()


class Unsecure(Module):
    verbose_name = _('Unsecure')
    verbose_name_plural = _('Unsecure')

    def count(self, qs):
        return qs.filter(is_secure=False).count()


class UniqueVisit(Module):
    verbose_name = _('Unique Visit')
    verbose_name_plural = _('Unique Visits')

    def count(self, qs):
        return qs.exclude(
            referer__startswith=settings.BASE_URL,
        ).count()


class UniqueVisitor(Module):
    verbose_name = _('Unique Visitor')
    verbose_name_plural = _('Unique Visitor')

    def count(self, qs):
        return qs.aggregate(Count('ip', distinct=True))['ip__count']


class User(Module):
    verbose_name = _('User')
    verbose_name_plural = _('User')

    def count(self, qs):
        return qs.exclude(user__isnull=False).count()


class UniqueUser(Module):
    verbose_name = _('Unique User')
    verbose_name_plural = _('Unique User')

    def count(self, qs):
        return qs.aggregate(Count('user', distinct=True))['user__count']



# -*- coding: utf-8 -*-
from django import template
from django.utils.http import urlquote

register = template.Library()


@register.simple_tag
def pie_chart(items, width=440, height=190):
    return '//chart.googleapis.com/chart?cht=p3&chd=t:{0}&chs={1}x{2}&chl={3}'.format(
        urlquote(','.join([str(item[1]) for item in items])),
        width,
        height,
        urlquote('|'.join([str(item[0]) for item in items])),
    )



# -*- coding: utf-8 -*-
import json
from socket import gethostbyaddr

import jsonfield
from django.conf import settings
from django.contrib.auth import get_user_model
from django.db import models
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
from six import python_2_unicode_compatible

from . import settings as request_settings
from .managers import RequestManager
from .utils import HTTP_STATUS_CODES, browsers, engines

AUTH_USER_MODEL = getattr(settings, 'AUTH_USER_MODEL', 'auth.User')


@python_2_unicode_compatible
class Request(models.Model):
    # Response information.
    response = models.SmallIntegerField(_('response'), choices=HTTP_STATUS_CODES, default=200)

    # Request information.
    method = models.CharField(_('method'), default='GET', max_length=7)
    path = models.CharField(_('path'), max_length=255)
    time = models.DateTimeField(_('time'), default=timezone.now, db_index=True)

    is_secure = models.BooleanField(_('is secure'), default=False)
    is_ajax = models.BooleanField(
        _('is ajax'),
        default=False,
        help_text=_('Wheather this request was used via javascript.'),
    )

    # User information.
    ip = models.GenericIPAddressField(_('ip address'))
    user = models.ForeignKey(AUTH_USER_MODEL, blank=True, null=True, verbose_name=_('user'), on_delete=models.SET_NULL)
    referer = models.URLField(_('referer'), max_length=255, blank=True, null=True)
    user_agent = models.CharField(_('user agent'), max_length=255, blank=True, null=True)
    language = models.CharField(_('language'), max_length=255, blank=True, null=True)

    # Add Get and Post params
    get_params = models.CharField(max_length=512, blank=True, default="")
    post_params = jsonfield.JSONField(blank=True, null=True, default="{}")

    objects = RequestManager()

    class Meta:
        app_label = 'request'
        verbose_name = _('request')
        verbose_name_plural = _('requests')
        ordering = ('-time',)

    def __str__(self):
        return '[{0}] {1} {2} {3}'.format(self.time, self.method, self.path, self.response)

    def get_user(self):
        return get_user_model().objects.get(pk=self.user_id)

    def from_http_request(self, request, response=None, commit=True):
        # Request information.
        self.method = request.method
        self.path = request.path[:255]

        self.is_secure = request.is_secure()
        self.is_ajax = request.is_ajax()

        # User information.
        self.ip = request.META.get('REMOTE_ADDR', '')
        self.referer = request.META.get('HTTP_REFERER', '')[:255]
        self.user_agent = request.META.get('HTTP_USER_AGENT', '')[:255]
        self.language = request.META.get('HTTP_ACCEPT_LANGUAGE', '')[:255]

        # Save get and post params
        self.get_params = "&".join([f"{k}={v}" for k, v in request.GET.items()])
        try:
            post_data = json.loads(request.body)
        except:
            post_data = {}
        self.post_params = post_data

        if hasattr(request, 'user') and hasattr(request.user, 'is_authenticated'):
            is_authenticated = request.user.is_authenticated
            if is_authenticated:
                self.user = request.user

        if response:
            self.response = response.status_code

            if (response.status_code == 301) or (response.status_code == 302):
                self.redirect = response['Location']

        if commit:
            self.save()

    @property
    def browser(self):
        if not self.user_agent:
            return

        if not hasattr(self, '_browser'):
            self._browser = browsers.resolve(self.user_agent)
        return self._browser[0]

    @property
    def keywords(self):
        if not self.referer:
            return

        if not hasattr(self, '_keywords'):
            self._keywords = engines.resolve(self.referer)
        if self._keywords:
            return ' '.join(self._keywords[1]['keywords'].split('+'))

    @property
    def hostname(self):
        try:
            return gethostbyaddr(self.ip)[0]
        except Exception:  # socket.gaierror, socket.herror, etc
            return self.ip

    def save(self, *args, **kwargs):
        if not request_settings.LOG_IP:
            self.ip = request_settings.IP_DUMMY
        elif request_settings.ANONYMOUS_IP:
            parts = self.ip.split('.')[0:-1]
            parts.append('1')
            self.ip = '.'.join(parts)
        if not request_settings.LOG_USER:
            self.user = None

        super(Request, self).save(*args, **kwargs)



from django.conf import settings

VALID_METHOD_NAMES = getattr(
    settings,
    'REQUEST_VALID_METHOD_NAMES',
    ('get', 'post', 'put', 'delete', 'head', 'options', 'trace'),
)

ONLY_ERRORS = getattr(settings, 'REQUEST_ONLY_ERRORS', False)
IGNORE_AJAX = getattr(settings, 'REQUEST_IGNORE_AJAX', False)
IGNORE_IP = getattr(settings, 'REQUEST_IGNORE_IP', tuple())
LOG_IP = getattr(settings, 'REQUEST_LOG_IP', True)
IP_DUMMY = getattr(settings, 'REQUEST_IP_DUMMY', '1.1.1.1')
ANONYMOUS_IP = getattr(settings, 'REQUEST_ANONYMOUS_IP', False)
LOG_USER = getattr(settings, 'REQUEST_LOG_USER', True)
IGNORE_USERNAME = getattr(settings, 'REQUEST_IGNORE_USERNAME', tuple())
IGNORE_PATHS = getattr(settings, 'REQUEST_IGNORE_PATHS', tuple())
IGNORE_USER_AGENTS = getattr(settings, 'REQUEST_IGNORE_USER_AGENTS', tuple())

TRAFFIC_MODULES = getattr(settings, 'REQUEST_TRAFFIC_MODULES', (
    'request.traffic.UniqueVisitor',
    'request.traffic.UniqueVisit',
    'request.traffic.Hit',
))

PLUGINS = getattr(settings, 'REQUEST_PLUGINS', (
    'request.plugins.TrafficInformation',
    'request.plugins.LatestRequests',

    'request.plugins.TopPaths',
    'request.plugins.TopErrorPaths',
    'request.plugins.TopReferrers',
    'request.plugins.TopSearchPhrases',
    'request.plugins.TopBrowsers',
))

try:
    from django.http import HttpRequest
    from django.contrib.sites.shortcuts import get_current_site
    BASE_URL = getattr(settings, 'REQUEST_BASE_URL', 'http://{0}'.format(get_current_site(HttpRequest()).domain))
except Exception:
    BASE_URL = getattr(settings, 'REQUEST_BASE_URL', 'http://127.0.0.1')



# -*- coding: utf-8 -*-
import json
from datetime import date, timedelta
from functools import update_wrapper

from django.conf.urls import url
from django.contrib import admin
from django.http import HttpResponse
from django.shortcuts import render
from django.utils.html import format_html
from django.utils.translation import ugettext_lazy as _

from .models import Request
from .plugins import plugins
from .traffic import modules


class RequestAdmin(admin.ModelAdmin):
    list_display = ('time', 'path', 'response', 'method', 'request_from')
    fieldsets = (
        (_('Request'), {
            'fields': ('method', 'path', 'time', 'is_secure', 'is_ajax')
        }),
        (_('Response'), {
            'fields': ('response',)
        }),
        (_('User info'), {
            'fields': ('referer', 'user_agent', 'ip', 'user', 'language')
        })
    )
    raw_id_fields = ('user',)
    readonly_fields = ('time',)

    def get_queryset(self, request):
        return super(RequestAdmin, self).get_queryset(request).select_related('user')

    def request_from(self, obj):
        if obj.user_id:
            return format_html(
                '<a href="?user={0}" title="{1}">{2}</a>',
                obj.user_id,
                _('Show only requests from this user.'),
                obj.user,
            )
        return format_html(
            '<a href="?ip={0}" title="{1}">{0}</a>',
            obj.ip,
            _('Show only requests from this IP address.'),
        )
    request_from.short_description = 'From'
    request_from.allow_tags = True

    def get_urls(self):
        def wrap(view):
            def wrapper(*args, **kwargs):
                return self.admin_site.admin_view(view)(*args, **kwargs)
            return update_wrapper(wrapper, view)

        info = (self.model._meta.app_label, self.model._meta.model_name)
        return [
            url(r'^overview/$', wrap(self.overview), name='{0}_{1}_overview'.format(*info)),
            url(r'^overview/traffic/$', wrap(self.traffic), name='{0}_{1}_traffic'.format(*info)),
        ] + super(RequestAdmin, self).get_urls()

    def overview(self, request):
        qs = Request.objects.this_month()
        for plugin in plugins.plugins:
            plugin.qs = qs

        return render(
            request,
            'admin/request/request/overview.html',
            {
                'title': _('Request overview'),
                'plugins': plugins.plugins,
            }
        )

    def traffic(self, request):
        try:
            days_count = int(request.GET.get('days', 30))
        except ValueError:
            days_count = 30

        if days_count < 10:
            days_step = 1
        elif days_count < 60:
            days_step = 2
        else:
            days_step = 30

        days = [date.today() - timedelta(day) for day in range(0, days_count + 1, days_step)]
        days_qs = [(day, Request.objects.day(date=day)) for day in days]
        return HttpResponse(json.dumps(modules.graph(days_qs)), content_type='text/javascript')


admin.site.register(Request, RequestAdmin)



# -*- coding: utf-8 -*-
from django.db.models import Count
from django.template.loader import render_to_string

from . import settings
from .models import Request
from .traffic import modules
from .utils import get_verbose_name


def set_count(items):
    '''
    This is similar to "set", but this just creates a list with values.
    The list will be ordered from most frequent down.

    Example:
        >>> inventory = ['apple', 'lemon', 'apple', 'orange', 'lemon', 'lemon']
        >>> set_count(inventory)
        [('lemon', 3), ('apple', 2), ('orange', 1)]
    '''
    item_count = {}
    for item in items:
        if not item:
            continue
        if item not in item_count:
            item_count[item] = 0
        item_count[item] += 1

    items = [(v, k) for k, v in item_count.items()]
    items.sort()
    items.reverse()

    return [(k, v) for v, k in items]


class Plugins(object):
    def load(self):
        from importlib import import_module
        from django.core import exceptions

        self._plugins = []
        for module_path in settings.PLUGINS:
            try:
                dot = module_path.rindex('.')
            except ValueError:
                raise exceptions.ImproperlyConfigured('{0} isn\'t a plugin'.format(module_path))
            plugin, plugin_classname = module_path[:dot], module_path[dot + 1:]

            try:
                mod = import_module(plugin)
            except ImportError as e:
                raise exceptions.ImproperlyConfigured('Error importing plugin {0}: "{1}"'.format(plugin, e))

            try:
                plugin_class = getattr(mod, plugin_classname)
            except AttributeError:
                raise exceptions.ImproperlyConfigured('Plugin "{0}" does not define a "{1}" class'.format(
                    plugin,
                    plugin_classname,
                ))

            self._plugins.append(plugin_class())

    def plugins(self):
        if not hasattr(self, '_plugins'):
            self.load()
        return self._plugins
    plugins = property(plugins)


plugins = Plugins()


class Plugin(object):
    def __init__(self):
        self.module_name = self.__class__.__name__

        if not hasattr(self, 'verbose_name'):
            self.verbose_name = get_verbose_name(self.module_name)

    def template_context(self):
        return {}

    def render(self):
        templates = [
            'request/plugins/{0}.html'.format(self.__class__.__name__.lower()),
            'request/plugins/base.html',
        ]

        if hasattr(self, 'template'):
            templates.insert(0, self.template)

        kwargs = self.template_context()
        kwargs['verbose_name'] = self.verbose_name
        kwargs['plugin'] = self
        return render_to_string(templates, kwargs)


class LatestRequests(Plugin):
    def template_context(self):
        return {'requests': Request.objects.all()[:5]}


class TrafficInformation(Plugin):
    def template_context(self):
        INFO_TABLE = ('today', 'this_week', 'this_month', 'this_year', 'all')
        INFO_TABLE_QUERIES = [getattr(Request.objects, query, None)() for query in INFO_TABLE]

        return {
            'traffic': modules.table(INFO_TABLE_QUERIES)
        }


class TopPaths(Plugin):
    def queryset(self):
        return self.qs.filter(response__lt=400)

    def template_context(self):
        return {
            'paths': self.queryset().values('path').annotate(Count('path')).order_by('-path__count')[:10]
        }


class TopErrorPaths(TopPaths):
    template = 'request/plugins/toppaths.html'

    def queryset(self):
        return self.qs.filter(response__gte=400)


class TopReferrers(Plugin):
    def queryset(self):
        return self.qs.unique_visits().exclude(referer='')

    def template_context(self):
        return {
            'referrers': self.queryset().values('referer').annotate(Count('referer')).order_by('-referer__count')[:10]
        }


class TopSearchPhrases(Plugin):
    def template_context(self):
        return {
            'phrases': set_count(self.qs.search().only('referer').attr_list('keywords'))[:10]
        }


class TopBrowsers(Plugin):
    def template_context(self):
        return {
            'browsers': set_count(self.qs.only('user_agent').attr_list('browser'))[:5]
        }


class ActiveUsers(Plugin):
    def template_context(self):
        return {}



#!/usr/bin/env python
import request
from setuptools import setup

setup(
    name='django-request-wyldw',
    version=request.__version__,
    description=open('docs/description.txt').read(),
    long_description=open('docs/long_description.txt').read(),
    author='Kyle Fuller',
    author_email='kyle@fuller.li',
    maintainer='Mariusz Felisiak',
    maintainer_email='felisiak.mariusz@gmail.com',
    url=request.__URL__,
    download_url='https://pypi.org/project/django-request-wyldw/',
    packages=[
        'request',
        'request.migrations',
        'request.templatetags',
        'request.management',
        'request.management.commands',
    ],
    package_data={'request': [
        'templates/admin/request/*.html',
        'templates/admin/request/request/*.html',
        'templates/request/plugins/*.html',
        'static/request/js/*.js',
        'locale/*/LC_MESSAGES/*.*',
    ]},
    install_requires=[
        'Django>=1.10',
        'python-dateutil',
        'six',
        "django-jsonfield"
    ],
    license=request.__licence__,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.10',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Framework :: Django :: 2.1',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.6',
    ],
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('request', '0003_auto_20160331_1430'),
    ]

    operations = [
        migrations.AlterField(
            model_name='request',
            name='time',
            field=models.DateTimeField(db_index=True, default=django.utils.timezone.now, verbose_name='time'),
        ),
    ]



__version__ = '0.0.4'
__copyright__ = 'Copyright (c) 2009-2019, Kyle Fuller, Mariusz Felisiak'
__licence__ = 'BSD'
__author__ = [
    'Kyle Fuller <kyle@fuller.li>',
    'Jannis Leidel (jezdez)',
    'krisje8 <krisje8@gmail.com>',
    'Mariusz Felisiak <felisiak.mariusz@gmail.com>',
]
__URL__ = 'https://django-request.readthedocs.io/en/latest/'
